#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>

/* librna header files */
#include "fold.h"
#include "fold_vars.h"
#include "utils.h"
#include "pair_mat.h"
#include "alifold.h"

#include "Utils.h"





/**************************************************
 *
 * RNAfold for a given sequence.
 * \returns mfe
 **************************************************/
float utils_fold_seq(char* sequence)
{
  /**** Call RNAfold via librna ****/
  int seqLength=0;
  float mfe;

  dangles=MY_DANGLES;
  noLonelyPairs=1;
  seqLength = strlen(sequence);
  char *structure =  (char *)space((unsigned)seqLength+1);
  /* Make sure that sequence has U's instead of T's */
  t2u(sequence);
  mfe = fold(sequence, structure);

  /* Free everything */
  free_arrays();
  free(structure);
 
  return mfe;
}

/**************************************************
 *
 * As fold() in the Vianna library does handle 
 * T's different from U's, we replace all T's in 
 * an input sequence to U's when folding.
 *
 **************************************************/
void t2u(char* seq)
{
  while((*seq)!='\0'){
    if((*seq)=='T') (*seq)='U';
    else if((*seq)=='t') (*seq)='u';
    seq++;
  }
  return;
}

/**************************************************
 *
 * Free multidimensional array.
 * Arrays must end with a NULL pointer!
 **************************************************/
void utils_free_mcarray(char *array[])
{
  int i;
  for (i=0;array[i]!=NULL;i++){
    free(array[i]);
  }
  array=NULL;
  return;
}


/**************************************************
 *
 * Reads lines of arbitrary length from fp. 
 * Code taken from librna.
 *
 * Sets the line pointer always to NULL. Does not 
 * need to take care about this.
 * Removes line delimiter. 
 *
 **************************************************/
char *utils_getline(FILE *fp)
{
  char s[512], *line, *cp;
  
  line = NULL;
  do {
    if (fgets(s, 512, fp)==NULL) break;
    cp = strchr(s, '\n');
    if (cp != NULL) *cp = '\0';
    if (line==NULL)
      line = (char*)calloc(strlen(s)+1, sizeof(char));     /* I replaced space() with calloc(). */
    else
      line = (char*)xrealloc(line, strlen(s)+strlen(line)+1);
    strcat(line, s);
  } while(cp==NULL);
  
  return line;
}


/**************************************************
 *
 * Get the critical value for DUDA like 
 * partitioning of a cluster tree.
 * It just depends on a significance value alpha, which 
 * actually corresponds to a z-score. The second parameter
 * is the number N of data points.
 * In this implementation the data vectors are assumed to 
 * have dimension 1.
 *
 **************************************************/
double utils_get_duda_critical_value(double alpha, int N)
{
  double f = 2. - (16./pow((double)M_PI,2.));
  double critValue =  1. - (2./(double)M_PI) - (alpha * sqrt((double)f/(double)N));
  return critValue;
}
